////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// LCDHype LCD Driver DLL Source Code                                         //
//                                                                            //
// Version 1.3                                                                //
//                                                                            //
// Requirements: LCDHype Version 0.37 or above                                //
//               DriverLINX DLPortIO Driver installed                         //
//               (if not needed remove the imports)                           //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

#define Packet		62*10

#define doppelbild

#include "stdafx.h"
#include "lcdllc.h"
#include "windows.h"
#include "dlportio.h" //include DriverLINX DLPortIO
#include "serial.h"
#include "lcd.h"

            //LCD Driver DLL Identifier String
const char* ID            = "Benedikts 640x480 LCD Controller";

            //maximum Pixels of graphics mode, 76800 = 320x240
const int   MAX_RESPIXELS = 76800;
      
//Custom Char Data Structure
struct TCharData 
{ BYTE position;   //Position of custom char in ram
  BYTE data[8][8]; //custom char 8x8 pixel data matrix
};

//LCD Driver Information Structure
struct TDLLInfo
{
  char IDArray[256];          //holds identifier of DLL
  bool SupportGfxLCD;         //does this driver support graphical LCDs?
  bool SupportTxtLCD;         //does this driver support text
  bool SupportLightSlider;    //does this driver support the light control slider
  bool SupportContrastSlider; //does this driver support the contrast slider
  bool SupportOutports;		  //does this driver support outports for controlling external circuits
  BYTE CCharWidth;            //custom char width in pixels
  BYTE CCharHeight;           //custom char height in pixels
  BYTE FontPitch;             //fontpitch of LCD in pixels
};

//Structure for Symbol LCDs
struct TSymbolInfo
{ 
  bool SymbolData[20]; //is symbol x displayed or not?
};

//global storage for LCD propertys
bool  LCD_LightOn     = false;  //is lcd light on?
ULONG LCD_Port        = 0x378; //port where lcd is connected to
DWORD LCD_Exectime    = 5000;  //execution time of lcd in textmode
DWORD LCD_ExectimeGfx = 1000;    //execution time of lcd in graphics mode
DWORD LCD_Width       = 320;   //width of LCDs graphics mode in pixels
DWORD LCD_Height      = 240;   //height of LCDs graphics mode in pixels 
DWORD LCD_Cols        = 80;    //number of chars per line in text mode
DWORD LCD_Rows        = 60;   //number of LCD lines in text mode
LONG  LCD_FontPitch    = 8;    //fontpitch of LCD in pixels   

//wiring bytes used for writing data to lpt port (original wiring)
BYTE CMD_High   = 5;
BYTE CMD_Low    = 1;
BYTE DATA_High  = 13;
BYTE DATA_Low   = 9;
BYTE Light_Byte = 2;

//function used to send data to lpt port based on lcd controller
void Send(ULONG Port, 
          BYTE High,
          BYTE Low,
          BYTE Value,
          DWORD Exectime)
{ 
/*  DWORD i;

 
  DlPortWritePortUchar(LCD_Port+2,High+((BYTE)(!LCD_LightOn))*Light_Byte);
  DlPortWritePortUchar(LCD_Port,Value);
  for (i=0;i<=Exectime;i++) { DlPortReadPortUchar(0x220); };
  DlPortWritePortUchar(LCD_Port+2,Low+((BYTE)(!LCD_LightOn))*Light_Byte);
  for (i=0;i<=Exectime;i++) { DlPortReadPortUchar(0x220); };*/
}

//function used to create a lcd driver config dialog
//dont export this function if you dont want to create config dialog
void _stdcall LCD_ConfigDialog()
{ };

//function used to send a byte to lcd controller
void _stdcall LCD_SendToController(BYTE Value)
{


 };

//function used to send a byte to lcd memory
void _stdcall LCD_SendToMemory(BYTE Value)
{
#ifdef Textmode
	unsigned char temp[5],i=0;
	temp[i++]=Escape;		//Escape Command
	temp[i++]=192;			//Text Wriet Command
	if (Value==Escape)		
		temp[i++]=Escape;
	temp[i++]=Value;		//Value
	SendString(temp, i);	//Send to LCD
#endif
 };

//function used to send gfx data to graphical lcd
void _stdcall LCD_SendToGfxMemory(BYTE Pixels[MAX_RESPIXELS], //pixel color data
                                  WORD x1,                    //refresh area x1,y1,x2,y2
                                  WORD y1, 
                                  WORD x2,
                                  WORD y2,
                                  bool inverted               //output of gfx data inverted?
                                 )
{	static int area,la;
	int x, y,xo,yo,i,j;
	unsigned char c;
	unsigned char temp[Packet+20];

	if (!Pixels[1])				//Erkennung der 4 Teilbilder
	{	area=0;
		xo=0;
		yo=0;
	}
	else if (!Pixels[2])
	{	area=1;
		xo=320;
		yo=0;
	}
	else if (!Pixels[3])
	{	area=2;
		xo=0;
		yo=240;
	}
	else if (!Pixels[4])
	{	area=3;
		xo=320;
		yo=240;
	}
	else
		return;


#ifdef doppelbild
	if (la==area)				//Doppelbilder vermeiden
		return;

	la=area;					//Zuletzt gesendetes Bild merken
#endif

	j=0;
	for (y=0; y<240; y++)
	{	
		temp[j++]=Escape;
		temp[j++]=1;
		if (((char)((y+yo)%240))==Escape)
			temp[j++]=Escape;
		temp[j++]=((char)((y+yo)%240));
		if (((char)(((y+yo)/240)*128+(xo/8)))==Escape)
			temp[j++]=Escape;
		temp[j++]=((char)(((y+yo)/240)*128+(xo/8)));
		if (j>=Packet)
			{SendString(temp, j);
			j=0;}
		for (x=0; x<320; x+=8)		//Bildzeile zusammenfassen
		{c=0;
			for (i=0; i<8;i++)
				c+=Pixels[y*320+x+i]*(1<<(7-i));
			if (c==Escape)
				temp[j++]=Escape;
			if (j>=Packet)
				{SendString(temp, j);
				j=0;}
				temp[j++]=c;
			if (j>=Packet)
				{SendString(temp, j);
				j=0;}
			}
	}
	SendString(temp, j);
/*

	for (y=0; y<240; y++)
		for (x=0; x<320; x++)		
		{
			if (inverted)
				lcdbuffer[y+yo][x+xo]=1-*Pixels++;		//copy inverted LCDHype data to LCD buffer
			else
				lcdbuffer[y+yo][x+xo]=*Pixels++;			//copy LCDHype data to LCD buffer
		}
		transfer (xo,yo,320+xo,240+yo);		//send part of lcd buffer to LCD
		//transfer (0,0,640,480);			//send part of lcd buffer to LCD

	area++;

	area&=3;*/


 };

//function to set the output address in text mode
void _stdcall LCD_SetOutputAddress(DWORD x, DWORD y)
{
 setcursor (x, y*8);
 };

//function to write a custom char to ram
void _stdcall LCD_SetCGRAMChar(TCharData cdata)
{ };

//function to get the position of custom char
//IN : custom char position based on HD44780 LCDs (0...)
//OUT: custom char position converted to current lcd controllers position
//
//Example : HD44780, first custom char is at position 0
//          T6963c , first custom char is at position 128
//
//          function LCD_GetCGRAMChar gets 1 and returns 128
BYTE _stdcall LCD_GetCGRAMChar(BYTE position)
{ return position; }; 

//function used to initialize LCD
void _stdcall LCD_Init(void)
{ 
};

//function used to clean up driver usage, called when DLL is closed.
void _stdcall LCD_CleanUp()
{ 
	uninitlcd();
	Uninit_Serial();
};

//function used to indicate that the lcd can receive data
//commonly used when interfacing serial lcds else return value true
bool _stdcall LCD_IsReadyToReceive()
{ return true; }; 

//function used to set the lcd propertys, so that the lcd driver can work correctly
//commonly used when supporting more than one lcd type in one driver
void _stdcall LCD_SetIOPropertys(char* Port,                //port lcd is connected to
								 DWORD Exectime,            //execution time for text mode
								 DWORD ExectimeGfx,         //execution time for gfx mode
								 DWORD X,                   //cols in text mode
								 DWORD Y,                   //rows in text mode
								 DWORD GX,                  //width in pixels in gfx mode
								 DWORD GY,                  //height in pixels in gfx mode
								 bool  LightOn,             //is light control enabled?
								 BYTE  LightSliderValue,    //brightness value from light slider
								 bool  ContrastOn,          //is contrast control enabled?
								 BYTE  ContrastSliderValue, //contrast value from contrast slider
								 DWORD Outports,            //32bit = 32 outports, bit=0=off, bit=1=on 
								 bool  UnderlineMode,       //is underline mode in text mode enabled?
								 bool  UnderlineOutput      //display underlined text?
								)
{	static bool initialised;
	if (!initialised)						//Wurde bereits initialisiert ?
	{
		if (Serial_Init(Port, 3000000))		//COM Port ffnen
			return;
		initlcd();							//LCD Initialisieren
		initialised=true;
	}

	if (ContrastOn)
		setpwm(ContrastSliderValue);	
};

//function used to interface symbols of a symbol lcd
void _stdcall LCD_Customize(TSymbolInfo SymData)
{ };

//function used to return information about the dll to the calling process
TDLLInfo _stdcall DLL_GetInfo()
{ 
  TDLLInfo result;  
  result.CCharWidth  = 8;       
  result.CCharHeight = 8;
  result.FontPitch = 8;
#ifdef Textmode
  result.SupportGfxLCD = false;
  result.SupportTxtLCD = true;
#else
  result.SupportGfxLCD = true;
  result.SupportTxtLCD = false;
#endif  
  result.SupportLightSlider = false;
  result.SupportContrastSlider = true;
  result.SupportOutports = false;

  ZeroMemory(&result.IDArray,256);
  CopyMemory(&result.IDArray[0],&ID[0],strlen(ID));

  return result;
};

